<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Matt Parmelee <mparmelee@interworx.com>
 * @date    Jul 21, 2011
 */

/**
 * Autoset Spamprefs plugin.
 *
 * Automatically sets SpamAssassin preferences on creation of a Siteworx account.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_AutosetSpamprefs extends Plugin
{

    /**
     * Post action.
     *
     * @param string        $ctrl_act Controller_Name:action
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($ctrl_act === 'Ctrl_Nodeworx_Siteworx:addCommit') {
            if ($Ctrl->isViewSuccess()) {
                $domain = $params->getValue('domainname');
                $input = array( 'enable_spam' => $this->getDatastoreVar('enable_spam'),
                        'dropscore' => $this->getDatastoreVar('dropscore'),
                        'spamscore' => $this->getDatastoreVar('spamscore'),
                        'rewrite_subject' =>
                        $this->getDatastoreVar('rewrite_subject'),
                        'report_safe' => $this->getDatastoreVar('report_safe') );

                $Reply = IW::FC()->routeFromPHP(
                    'Ctrl_Siteworx_EmailSpamprefs',
                    'editCommit',
                    $input,
                    IW::NW()->getSiteWorx($domain)
                );
            }
        }
        return;
    }

    /**
     * Initialize this plugin's form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $Group = $Form->addGroup('extra_options', '##LG_EXTRA_PLUGIN_SETTINGS##');

        //enable spam
        $DS = DataSource_Boolean::factory(DataSource_Boolean::ON_OFF);

        $enable_spam = $this->getDatastoreVar('enable_spam');
        $Group->addInput(new Input_Select('enable_spam', $enable_spam))
        ->setLabel('##LG_SPAMASSASSIN_STATUS##')
        ->setDataSource($DS);

        //drop score
        $DS = new DataSource_Array(
            array( 'no' ),
            array( '##LG_DELIVER_TO_INBOX_OR_SPAM_FOLDER##' )
        );

        $CustomInput = new Input_Number('dropscore_custom');
        $CustomInput->setRange(0, 999);
        $CustomInput->setLengthRange(1, 6);
        $CustomInput->setLabel('##LG_DELETE_SPAM_SCORED_HIGHER_THAN_THIS_VALUE##');

        $dropscore = $this->getDatastoreVar('dropscore');
        $Group->addInput(new Input_SetCustom(
            'dropscore',
            $DS,
            $CustomInput,
            $dropscore
        ))
        ->setLabel('##LG_SPAMDELIVERY##');

        //spam score
        $values         = array( 5, 7, 10 );
        $display_values = array( '5 (##LG_AGGRESSIVE##)',
                             '7 (##LG_LESSAGGRESSIVE##)',
                             '10 (##LG_CONSERVATIVE##)' );

        $DS = new DataSource_Array($values, $display_values);

        $CustomInput = new Input_Number('spamscore_custom');
        $CustomInput->setRange(0, 999);
        $CustomInput->setLabel('(##LG_CUSTOM##)');

        $spamscore = $this->getDatastoreVar('spamscore');
        $Group->addInput(new Input_SetCustom(
            'spamscore',
            $DS,
            $CustomInput,
            $spamscore
        ))
        ->setLabel('##LG_SPAMSCORETHRESHOLD##');

        //rewrite subject
        $CustomInput = new Input_String('rewrite_subject_custom', '[SPAM]');
        $CustomInput->setLabel('(##LG_CUSTOM##)');

        $DS = new DataSource_Array(
            array( 'no' ),
            array( '##LG_DONOTREWRITESUBJECT##' )
        );

        $rewrite_subject = $this->getDatastoreVar('rewrite_subject');
        $Group->addInput(new Input_SetCustom(
            'rewrite_subject',
            $DS,
            $CustomInput,
            $rewrite_subject
        ))
        ->setLabel('##LG_REWRITESUBJECT##');

        //report safe
        $values         = array( 0, 1, 2 );
        $display_values = array( '##LG_REPORT_SAFE_0##',
                             '##LG_REPORT_SAFE_1##',
                             '##LG_REPORT_SAFE_2##' );

        $DS = new DataSource_Array($values, $display_values);

        $report_safe = $this->getDatastoreVar('report_safe');
        $Group->addInput(new Input_Set('report_safe', $report_safe))
        ->setDataSource($DS)
        ->setLabel('##LG_MESSAGEATTACHMENTPOLICY##');
    }

    /**
     * Process the form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $this->setDatastoreVar('enable_spam', $Form->getValue('enable_spam'));
        $this->setDatastoreVar('dropscore', $Form->getValue('dropscore'));
        $this->setDatastoreVar('spamscore', $Form->getValue('spamscore'));
        $this->setDatastoreVar(
            'rewrite_subject',
            $Form->getValue('rewrite_subject')
        );
        $this->setDatastoreVar('report_safe', $Form->getValue('report_safe'));
    }
}
