<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 14, 2014
 */

/**
 * Class Plugin_DomainEmailCreate.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_DomainEmailCreate extends Plugin
{
    protected $_enabled_by_default = true;

    /**
     * Get category.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::EMAIL;
    }

    /**
     * Post action.
     *
     * @param string        $ctrl_act
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($params instanceof Form === false) {
            return;
        }
        if (!$Ctrl->isViewSuccess()) {
            return;
        }
        if (IW::Env()->isNode()) {
            return;
        }

        $Form = $params;

        switch ($ctrl_act) {
            case 'Ctrl_Nodeworx_Siteworx:addCommit':
                $domain = $Form->getValue('master_domain');
                break;

            case 'Ctrl_Siteworx_DomainsSlave:addCommit':
                $domain = $Form->getValue('domain');
                break;

            default:
                return;
        }//end switch

        $this->_addEmailAccounts($domain);
    }

    /**
     * Initialize this plugin's form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $Group = $Form->addGroup('extra_options', 'Extra Plugin Settings');

        $boxes = $this->getDatastoreVar('accounts');

        $Group->addInput(new Input_String('accounts', $boxes))
        ->setMultipleAllowed(true)
        ->addValidator(new IWorx_Validate_EmailUsername())
        ->setRequired(false);
    }

    /**
     * Process the form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $this->setDatastoreVar('accounts', $Form->getValue('accounts'));
    }

    /**
     * Add email boxes.
     *
     * @param string $domain
     * @throws Exception       Sometimes.
     * @throws IWorx_Exception If route errors.
     */
    private function _addEmailAccounts($domain)
    {
        $SW    = new SiteWorx($domain);
        $quota = $SW->getIncomingMailDriver()->getBoxesAllowed();
        $used  = $SW->getIncomingMailDriver()->getBoxesUsed();
        $boxes = $this->getDatastoreVar('accounts');
        $boxes = (array) $boxes;

        if ($used + count($boxes) > $quota) {
            IWorxLog::log('Skipping automatic email account creation for ' . $domain);
            IWorxLog::log('Quota would have been exceeded');
            return;
        }

        $passwd = IWorxUtil::generatePassword();

        foreach ($boxes as $box) {
            $input = array( 'username' => $box,
                      'password' => $passwd,
                      'confirm_password' => $passwd );

            $Reply = IW::FC()->routeFromPHP(
                'Ctrl_Siteworx_EmailBox',
                'addCommit',
                $input,
                $SW
            );

            if ($Reply->wasSuccessful() === false) {
                IWorxLog::log("Creating box {$box}@{$domain} failed");
                IWorxLog::log($Reply->getErrors());
            }
        }
    }

    /**
     * Set some defaults.
     */
    public function install()
    {
        $this->setDatastoreVar('accounts', 'postmaster');
    }
}
