<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Sep 21, 2009
 */

/**
 * R1Soft siteworx redirect.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Ctrl_Siteworx_R1soft extends Ctrl_Siteworx_Plugin
{

    /**
     * Default action.
     */
    public function indexAction()
    {
        $blocked = $this->_getPlugin()->userIsBlocked(IW::SW()->getUniqname());
        if ($blocked === true) {
            throw new IWorx_Exception_Permission('Permission denied');
        }

        if ($this->_getPlugin()->getCdpVer() === '2') {
            $email     = IW::SW()->getWorkingUser()->getEmail();
            $serverurl = $this->_getPlugin()->getVar('serverurl');
            $hostname  = $this->_getPlugin()->getVar('hostname');
            $this->getView()->assign('template', 'redirect-cdp2');
            $this->getView()->assign('title', 'R1Soft Backups');
            $this->getView()->assign('serverurl', $serverurl);
            $this->getView()->assign('hostname', $hostname);
            $this->getView()->assign('remoteuser', $email);
            $this->getView()->assign('sessionid', session_id());
        } elseif ($this->_getPlugin()->getCdpVer() === '3') {
            $uniq      = IW::SW()->getUniqname();
            $serverurl = $this->_getPlugin()->getDatastoreVar('cdp_server_url');
            $cp_id     = $this->_getPlugin()->getDatastoreVar('cdp_control_panel_id');
            $this->getView()->assign('template', 'redirect-cdp3');
            $this->getView()->assign('title', 'R1Soft Backups');
            $this->getView()->assign('serverurl', $serverurl);
            $this->getView()->assign('cpID', $cp_id);
            $this->getView()->assign('remoteuser', $uniq);
            $this->getView()->assign('sessionid', session_id());
        } else {
            throw new IWorx_Exception_Permission('Unknown CDP version');
        }//end if
    }

    /**
     * Get auth information.
     */
    public function listAuthInfoAction()
    {
        $blocked = $this->_getPlugin()->userIsBlocked(IW::SW()->getUniqname());

        if ($blocked === false) {
            $data = array( 'uid' => ShellAccount::getUID(IW::SW()->getUniqname()),
                     'gid' => ShellAccount::getGID(IW::SW()->getUniqname()),
                     'packageroot' => IW::SW()->getPackageRoot(),
                     'workinguser' => IW::SW()->getWorkingUser()->getEmail(),
                     'unixuser' => IW::SW()->getUniqname() );
        } else {
            $data = array( 'blocked' => '1' );
        }

        $this->_displayPayload(new Payload($data));
    }
}
