<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2016 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    interworx-cli-core
 * @subpackage CLI
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 24, 2016
 */

namespace Interworx\CliCore;

/**
 * Expands ./, ../, ~, etc to a real path.
 * TODO: This is IWorx_Filter_FilePathCli.  Should the Filters be moved into a
 * composer-like environment, this should be removed and made to use that.
 *
 * @package    interworx-cli-core
 * @subpackage Filter
 */
class PathFilter implements \Zend_Filter_Interface
{
    /**
     * Filter the value, expanding linux path shorthand.
     *
     * @param string $value
     * @return string
     */
    public function filter($value)
    {
        if (strpos($value, '~/') === 0) {
            // Starts with ~/
            $value = $this->_expandMyHomeDir($value);
        } else {
            if (strpos($value, '~') === 0) {
                // Starts with ~, but then contains a user name
                $value = $this->_expandSomeoneElsesHomeDir($value);
            }
        }

        return $this->_realpath($value);
    }

    /**
     * Expand the homedir of the user that launched the process.
     * So, if you were root, ~/whatever would likely expand to /root/whatever.
     *
     * @param string $value
     * @return string
     */
    private function _expandMyHomeDir($value)
    {
        $user = getenv('IWORX_UNIXUSER');

        if (!$user) {
            // This is really only required for devs using .php files directly, not .pex
            $user = getenv('USER');
        }

        $value = str_replace('~/', "~{$user}/", $value);

        return $this->_expandSomeoneElsesHomeDir($value);
    }

    /**
     * Expand a homedir where the username is explicitly stated.
     * ~iworx, for example.
     *
     * @param string $value
     * @return string
     */
    private function _expandSomeoneElsesHomeDir($value)
    {
        if ($slash_position = strpos($value, '/')) {
            /*
             * We usually use === false, but in this case, 0 is unacceptable, too, so
             * I'm using PHP's falsey.
             */
            $end = $slash_position - 1;
        } else {
            $end = strlen($value) - 1;
        }

        $user = substr($value, 1, $end);

        $user_info = posix_getpwnam($user);

        return str_replace("~{$user}", $user_info['dir'], $value);
    }

    /**
     * Converts to the real path, resolving ./ and ../.
     *
     * @param string $value
     * @return string
     */
    private function _realpath($value)
    {
        /*
         * Only do the directory changing if the directory looks relative.
         */
        if ($value{0} !== '/') {
            if (defined('IWORX_START_DIR')) {
                // Need to chdir to the directory we launched from in order to resolve
                // a relative path
                $current_working_dir = getcwd();
                chdir(IWORX_START_DIR);
            }
        }

        $output = realpath($value);

        if ($value{0} !== '/') {
            if (defined('IWORX_START_DIR')) {
                // Put the old cwd back
                chdir($current_working_dir);
            }
        }

        if ($output !== false) {
            return $output;
        }

        // If the path was bullshit, other validators will kill it anyway.
        return $value;
    }
}

